-------------------------------
-- RogueRotationHelper | Created by TheoChao --
-------------------------------

RogueRotationHelper = LibStub("AceAddon-3.0"):NewAddon("RogueRotationHelper", "AceConsole-3.0", "AceEvent-3.0")



local ROGUE = 1
local DRUID = 2

local CombatEventHandlers = {}

local defaults = {
    profile = {
        ActiveFlag = true,
        SoundEnabled = true,
        TalentTree = 1,
        EnergyOffset = 5,
        RuptureTimer = 3,
        SliceTimer = 3,
        SuggestBackstab = true,
        SuggestTricks = true,
        SuggestFeint = false,
        SuggestVanish = false,
        SuggestRupture = true,
        SuggestArmorCP = 2,
        TrackToTT = true,
        TrackVendetta = true,
        TrackColdblood = true,
        TrackVanish = true,
        TrackRacial = true,
        UILocked = false,
        UIScale = 1,
        x = 0,
        y = 0,
        point = "CENTER",
        lastSpell = "",

        castSettings = {
            isCasting = false,
        },
        
        -- Create an Array of spells used within RogueRotationHelper's rotation algorithm.  it will map the spells to the sound to play when the spell is triggered
        SpellSounds = {
            ["Garrote"]             = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\three.mp3",
            ["Slice and Dice"]      = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\five.mp3",
            ["Vendetta"]            = "",
            ["Mutilate"]            = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\two.mp3",
            ["Backstab"]            = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\one.mp3",
            ["Envenom"]             = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\three.mp3",
            ["Rupture"]             = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\four.mp3",
            ["Cold Blood"]          = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\letter-g.mp3",
            ["Tricks of the Trade"] = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\letter-c.mp3",
            ["Vanish"]              = "",
            ["Revealing Strike"]    = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\one.mp3",
            ["Sinister Strike"]     = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\two.mp3",
            ["Eviscerate"]          = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\three.mp3",
            ["Hemorrhage"]          = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\two.mp3",
            ["Recuperate"]          = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\four.mp3",
            ["Ambush"]              = "Interface\\AddOns\\RogueRotationSuggestor\\Sounds\\two.mp3",
            ["Premeditation"]       = "",
            ["Shadowstep"]          = "",
            
            ["Kick"]                = "",
            ["Feint"]               = "Interface\\AddOns\\RogueRotationHelper\\Sounds\\letter-f.mp3",
            
        }
    },
    
}

-- this is the array for the description of the sound options
local soundOptionsArray = {
    "<None>",
    "<Ping>",
    "One",
    "Two",
    "Three",
    "Four",
    "Five",
    "The letter C",
    "The letter F",
    "The letter G",
}

-- this is the array for the sound files that match with the options (this must be in the same order as soundOptionsArray)
local soundOptionsMappings = {
    "",
    "Interface\\AddOns\\RogueRotationSuggestor\\Sounds\\pingit.mp3",
    "Interface\\AddOns\\RogueRotationHelper\\Sounds\\one.mp3",
    "Interface\\AddOns\\RogueRotationHelper\\Sounds\\two.mp3",
    "Interface\\AddOns\\RogueRotationHelper\\Sounds\\three.mp3",
    "Interface\\AddOns\\RogueRotationHelper\\Sounds\\four.mp3",
    "Interface\\AddOns\\RogueRotationHelper\\Sounds\\five.mp3",
    "Interface\\AddOns\\RogueRotationHelper\\Sounds\\letter-c.mp3",
    "Interface\\AddOns\\RogueRotationHelper\\Sounds\\letter-f.mp3",
    "Interface\\AddOns\\RogueRotationHelper\\Sounds\\letter-g.mp3",
}

local wowInterfaceConfig = {
    name = "Rogue Rotation Suggestor Options",
    handler = RogueRotationHelper,
    type = "group",
    args = {
    
        rotationFrame = {
            type = "group",
            inline = true,
            name = "Rotation",
            desc = "",
            order = 1,
            get = function(info) return RogueRotationHelper.db.profile[info[#info]] end,
            set = function(info, value)
                RogueRotationHelper.db.profile[info[#info]] = value
            end,
            args = {
                ActiveFlag = {
                    type = "toggle",
                    name = "Enable",
                    order = 0,
                    desc = "Enable the Rogue Rotation Suggestor",
                    width = "full",
                },
        
                SuggestBackstab = {
                    type = "toggle",
                    name = "Suggest Backstab",
                    order = 1,
                    desc = "Suggest backstab instead of Mutilate if target is bellow 35% health.",
                    width = "full",
                    
                },
                
                SuggestFeint = {
                    type = "toggle",
                    name = "Suggest Feint",
                    order = 2,
                    desc = "Suggest feint when off cooldown and nothing else to cast",
                    width = "full",
                },
            
                SuggestTricks = {
                    type = "toggle",
                    name = "Suggest Tricks of the Trade",
                    order = 3,
                    desc = "Suggest Tricks of the Trade when off cooldown and nothing else to cast",
                    width = "full",
                },

                SuggestRupture = {
                    type = "toggle",
                    name = "Suggest Rupture",
                    order = 4,
                    desc = "Suggest rupture in combat spec.",
                    width = "full",
                    
                },
                SuggestArmorCP = {
                    order = 5,
                    width = "full",
                    type = "range",
                    name = "Suggest Armor at CP",
                    desc = "Suggest Expose Armor when the target does not have an armor debuff and CP at least this many (combat spec).  Set to 6 to turn off",
                    min = 1,
                    max = 6,
                    step = 1,
                    bigStep = 1,
                },
                
                
            },
        },
                
        timingFrame = { 
            type = "group",
            inline = true,
            name = "Timing",
            desc = "",
            order = 2,
            get = function(info) return RogueRotationHelper.db.profile[info[#info]] end,
            set = function(info, value)
                RogueRotationHelper.db.profile[info[#info]] = value
            end,
            args = {
        
                RuptureTimer = {
                    type = "range",
                    name = "Rupture Refresh Timer",
                    desc = "Suggest Rupture when the targets Rupture debuff time is bellow or the same as the set amount of seconds.",
                    order = 1,
                    min = 0,
                    max = 6,
                    step = 1,
                    bigStep = 1,
                },
                
                SliceTimer = {
                    type = "range",
                    name = "Slice & Dice Refresh Timer",
                    desc = "Suggest S&D when the time is bellow or the same as the set amount of seconds.",
                    order = 2,
                    min = 0,
                    max = 6,
                    step = 1,
                    bigStep = 1,
                },
            
            },
        },
                
        
        soundFrame = {
            type = "group",
            inline = true,
            name = "Sound Cues",
            desc = "",
            order = 2,
            get = function(info) 
                    for k,v in pairs(soundOptionsMappings) do
                        if(RogueRotationHelper.db.profile.SpellSounds[info[#info]] == v) then
                            return k
                        end
                    end
                return 
                end,
            set = function(info,value)
                    RogueRotationHelper.db.profile.SpellSounds[info[#info]] = soundOptionsMappings[value] 
                    PlaySoundFile(RogueRotationHelper.db.profile.SpellSounds[info[#info]], "MASTER")
                end,
            
            args = {
                ["SoundEnabled"] = {
                    type = "toggle",
                    name = "Enable Sound Cues",
                    order = 0,
                    desc = "Enable the Rogue Rotation Suggestor sound cues",
                    width = "full",
                    get = function(info) return RogueRotationHelper.db.profile.SoundEnabled end,
                    set = function(info, value)
                        RogueRotationHelper.db.profile.SoundEnabled = value
                    end,
                },
                
                ["Mutilate"] = { type = "select", values = soundOptionsArray, name = "Mutilate", order = 0 },
                ["Backstab"] = { type = "select", values = soundOptionsArray, name = "Backstab", order = 1 },
                ["Rupture"] = { type = "select", values = soundOptionsArray, name = "Rupture", order = 2 },
                ["Slice and Dice"] = { type = "select", values = soundOptionsArray, name = "Slice and Dice", order = 3 },
                ["Envenom"] = { type = "select", values = soundOptionsArray, name = "Envenom", order = 4 },
                ["Cold Blood"] = { type = "select", values = soundOptionsArray, name = "Cold Blood", order = 5 },
                ["Garrote"] = { type = "select", values = soundOptionsArray, name = "Garrote", order = 6 },
                
                ["Feint"] = { type = "select", values = soundOptionsArray, name = "Feint", order = 7 },
                ["Tricks of the Trade"] = { type = "select", values = soundOptionsArray, name = "Tricks of the Trade", order = 8 },
                
                ["Sinister Strike"] = { type = "select", values = soundOptionsArray, name = "Sinister Strike", order = 9 },
                ["Revealing Strike"] = { type = "select", values = soundOptionsArray, name = "Revealing Strike", order = 10 },
                ["Eviscerate"] = { type = "select", values = soundOptionsArray, name = "Eviscerate", order = 11 },
                ["Hemorrhage"] = { type = "select", values = soundOptionsArray, name = "Hemorrhage", order = 12 },
                ["Recuperate"] = { type = "select", values = soundOptionsArray, name = "Recuperate", order = 13 },
                ["Ambush"] = { type = "select", values = soundOptionsArray, name = "Ambush", order = 14 },
                ["Premeditation"] = { type = "select", values = soundOptionsArray, name = "Premeditation", order = 15 },
                ["Shadowstep"] = { type = "select", values = soundOptionsArray, name = "Shadowstep", order = 16 },
                
            }
        },
        
        uiFrame = {
            type = "group",
            inline = true,
            name = "User Interface",
            desc = "",
            order = 3,
            args = {
                
                UILocked = {
                    type = "toggle",
                    name = "Lock",
                    order = 0,
                    desc = "Lock RogueRotationHelper's UI. This will also cause RogueRotationHelper's UI to become transparent.",
                    width = "full",
                    get = function(info) return RogueRotationHelper.db.profile.UILock end,
                    set = "ToggleUILocked",
                },
            
                UIScale = {
                    type = "range",
                    name = "UI Scale",
                    desc = "Toggle to increase/decrease the scale of RogueRotationHelper's UI.",
                    order = 1,
                    min = 0.5,
                    max = 1.5,
                    step = 0.1,
                    bigStep = 0.1,
                    get = function(info) return RogueRotationHelper.db.profile.UIScale end,
                    set = function(info, value)
                        RogueRotationHelper.db.profile.UIScale = value
                        RogueRotationHelper.displayFrame:SetScale(value)
                    end,
                },
                
            },
        },
        
    }
}




function RogueRotationHelper:OnEnable()
    local _, playerClass = UnitClass("player")
    if playerClass == "ROGUE" then
        self.db = LibStub("AceDB-3.0"):New("RogueRotationHelperDB", defaults, "Default")

        LibStub("AceConfig-3.0"):RegisterOptionsTable("RogueRotationHelper", wowInterfaceConfig)
        self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("RogueRotationHelper", "RogueRotationHelper")
        self:RegisterChatCommand("rogrot", "ChatCommand")
        --self:RegisterChatCommand("RogueRotationHelper", "ChatCommand")
    
        -- Create UI
        self:CreateUI()
        
        -- Load UI Settings
        self.displayFrame:SetScale(self.db.profile.UIScale)
        
        if self.db.profile.UILock then
            self.displayFrame:SetScript("OnMouseDown", nil)
            self.displayFrame:SetScript("OnMouseUp", nil)
            self.displayFrame:SetScript("OnDragStop", nil)
            self.displayFrame:SetBackdropColor(0, 0, 0, 0)
            self.displayFrame:EnableMouse(false)
        else
            self.displayFrame:SetScript("OnMouseDown", function(self) self:StartMoving() end)
            self.displayFrame:SetScript("OnMouseUp", function(self) self:StopMovingOrSizing() end)
            self.displayFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
            self.displayFrame:SetBackdropColor(0, 0, 0, .4)
            self.displayFrame:EnableMouse(true)
        end
        
        self.displayFrame:SetScript("OnUpdate", function(this, elapsed) self:OnUpdate(elapsed) end)
        
        -- Register Events
        self:RegisterEvent("PLAYER_TALENT_UPDATE")
        self:PLAYER_TALENT_UPDATE()
        
        
        self.combatFrame:SetScript("OnEvent", function(s,e,g,...) if( g=="target" ) then self:OnUpdateCast() end end) 
        --self.combatFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
        self.combatFrame:RegisterEvent("UNIT_SPELLCAST_START");
        self.combatFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE");
        self.combatFrame:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE");
        self.combatFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
    end
end



function RogueRotationHelper:PLAYER_TALENT_UPDATE()
    -- Check Vendetta is a useable spell
    local _, _, _, _, t = GetTalentInfo(1, 19) 
    local _, _, _, _, t2 = GetTalentInfo(2, 19) 
    local _, _, _, _, t3 = GetTalentInfo(3, 19) 
    if t == 1 then
        RogueRotationHelper.db.profile.TalentTree = 1
        --self.Spells["TalentTreeSpell1"] = self.Spells["Vendetta"]
        --self.Spells["TalentTreeSpell2"] = self.Spells["Cold Blood"]
        self.displayFrame:Show() 
    elseif t2 == 1 then
        RogueRotationHelper.db.profile.TalentTree = 2
        --self.Spells["TalentTreeSpell1"] = self.Spells["Killing Spree"]
        --self.Spells["TalentTreeSpell2"] = self.Spells["Adrenaline Rush"]
        self.displayFrame:Show()
    elseif t3 == 1 then
        RogueRotationHelper.db.profile.TalentTree = 3
        --self.Spells["TalentTreeSpell1"] = self.Spells["Shadow Dance"]
        --self.Spells["TalentTreeSpell2"] = self.Spells["Preparation"]
        self.displayFrame:Show()
    else
        self.displayFrame:Hide() 
        RogueRotationHelper.db.profile.TalentTree = 0
    end
    self.displayFrame_current.Border:SetTexture(0,0,0,0)
end


function RogueRotationHelper:OnUpdate(elapsed)

    if self.db.profile.ActiveFlag then
        local point,_,_,xPos,yPos = self.displayFrame:GetPoint()
        RogueRotationHelper.db.profile.x = xPos;
        RogueRotationHelper.db.profile.y = yPos;
        RogueRotationHelper.db.profile.point = point;
        
        if UnitName("target") == nil or UnitIsFriend("player","target") ~= nil or UnitHealth("target") == 0 or UnitGUID("target") == nil or (UnitAura("player", "Stealth") ~= nil and UnitPower("player")==UnitPowerMax("player") and RogueRotationSuggestor.db.profile.TalentTree ~= 3) then
            self.displayFrame_current:Hide()
            self.displayFrame_next:Hide()
        else
            self.displayFrame_current:Show()
            self.displayFrame_next:Show()
            self:CalculateRotation()
        end
    end
end



